==> spring_0.76b1_src.tar.bz2: spring_0.76b1/rts/System/Object.cpp <==
#include "StdAfx.h"
// Object.cpp: implementation of the CObject class.
//
//////////////////////////////////////////////////////////////////////

#include "Object.h"
#include "mmgr.h"
#include "creg/STL_Set.h"
#include "LogOutput.h"

CR_BIND(CObject, )

CR_REG_METADATA(CObject, (
//	CR_MEMBER(listening),
//	CR_MEMBER(listeners),
	CR_SERIALIZER(Serialize),
	CR_POSTLOAD(PostLoad)
	));

CObject* CObject::syncedObjects = 0;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CObject::CObject(EObjectType synced) : prev(NULL), next(NULL)
{
	/*if (synced == OT_Synced)*/ {
		// Push a synchronized object on front of the list.
		next = syncedObjects;
		syncedObjects = this;
		if (next)
			next->prev = this;
	}
}

CObject::~CObject()
{
	std::list<CObject*>::iterator di;
	for(di=listeners.begin();di!=listeners.end();++di){
		(*di)->DependentDied(this);
		ListErase<CObject*>((*di)->listening, this);
	}
	for(di=listening.begin();di!=listening.end();++di){
		ListErase<CObject*>((*di)->listeners, this);
	}

	// remove from list of synced objects
	if (IsSynchronized()) {
		if (prev)
			prev->next = next;
		else
			syncedObjects = next;
		if (next)
			next->prev = prev;
	}
}

void CObject::Serialize(creg::ISerializer *s)
{
	if (s->IsWriting ()) {
		int size=0;
		for (std::list<CObject*>::iterator i=listening.begin();i!=listening.end();++i) {
			if ((*i)->GetClass()!=CObject::StaticClass())
				size++;
		}
		s->Serialize (&size, sizeof(int));
		for (std::list<CObject*>::iterator i=listening.begin();i!=listening.end();++i) {
			if ((*i)->GetClass()!=CObject::StaticClass())
				s->SerializeObjectPtr((void **)&*i,(*i)->GetClass());
			else
				logOutput.Print("Death dependance not serialized in %s",this->GetClass()->name.c_str());
		}
	} else {
		int size;
		s->Serialize (&size, sizeof(int));
		for (int i=0;i<size;i++) {
			std::list<CObject*>::iterator itm=listening.insert(listening.end(),0);
			s->SerializeObjectPtr((void **)&*itm,0);
		}
	}
}

void CObject::PostLoad()
{
	for (std::list<CObject*>::iterator i=listening.begin();i!=listening.end();i++)
		(*i)->listeners.insert((*i)->listeners.end(),this);
}

void CObject::DependentDied(CObject* o)
{
//	MessageBox(0,"wrong","ag",0);
}

void CObject::AddDeathDependence(CObject *o)
{
	o->listeners.insert(o->listeners.end(),this);
	listening.insert(listening.end(),o);
}

void CObject::DeleteDeathDependence(CObject *o)
{
	//note that we can be listening to a single object from several different places (like curreclaim in CBuilder and lastAttacker in CUnit, grr) so we should only remove one of them
	ListErase<CObject*>(listening, o);
	ListErase<CObject*>(o->listeners, this);
}

==> spring_0.77b5_src.tar.bz2: spring_0.77b5/rts/System/Object.cpp <==
#include "StdAfx.h"
// Object.cpp: implementation of the CObject class.
//
//////////////////////////////////////////////////////////////////////

#include "Object.h"
#include "mmgr.h"
#include "creg/STL_Set.h"
#include "LogOutput.h"

CR_BIND(CObject, )

CR_REG_METADATA(CObject, (
//	CR_MEMBER(listening),
//	CR_MEMBER(listeners),
	CR_SERIALIZER(Serialize),
	CR_POSTLOAD(PostLoad)
	));

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CObject::CObject()
{
}

CObject::~CObject()
{
	std::list<CObject*>::iterator di;
	for(di=listeners.begin();di!=listeners.end();++di){
#ifdef USE_MMGR
 		m_setOwner(__FILE__, __LINE__, __FUNCTION__);
#endif
		(*di)->DependentDied(this);
#ifdef USE_MMGR
		m_setOwner(__FILE__, __LINE__, __FUNCTION__);
#endif
		ListErase<CObject*>((*di)->listening, this);
	}
	for(di=listening.begin();di!=listening.end();++di){
#ifdef USE_MMGR
		m_setOwner(__FILE__, __LINE__, __FUNCTION__);
#endif
		ListErase<CObject*>((*di)->listeners, this);
	}
#ifdef USE_MMGR
	m_resetGlobals();
#endif
}

void CObject::Serialize(creg::ISerializer *s)
{
	if (s->IsWriting ()) {
		int size=0;
		for (std::list<CObject*>::iterator i=listening.begin();i!=listening.end();++i) {
			if ((*i)->GetClass()!=CObject::StaticClass())
				size++;
		}
		s->Serialize (&size, sizeof(int));
		for (std::list<CObject*>::iterator i=listening.begin();i!=listening.end();++i) {
			if ((*i)->GetClass()!=CObject::StaticClass())
				s->SerializeObjectPtr((void **)&*i,(*i)->GetClass());
			else
				logOutput.Print("Death dependance not serialized in %s",this->GetClass()->name.c_str());
		}
	} else {
		int size;
		s->Serialize (&size, sizeof(int));
		for (int i=0;i<size;i++) {
			std::list<CObject*>::iterator itm=listening.insert(listening.end(),0);
			s->SerializeObjectPtr((void **)&*itm,0);
		}
	}
}

void CObject::PostLoad()
{
	for (std::list<CObject*>::iterator i=listening.begin();i!=listening.end();i++) {
#ifdef USE_MMGR
		m_setOwner(__FILE__, __LINE__, __FUNCTION__);
#endif
		(*i)->listeners.insert((*i)->listeners.end(),this);
	}
#ifdef USE_MMGR
	m_resetGlobals();
#endif
}

void CObject::DependentDied(CObject* o)
{
//	MessageBox(0,"wrong","ag",0);
}

void CObject::AddDeathDependence(CObject *o)
{
#ifdef USE_MMGR
	m_setOwner(__FILE__, __LINE__, __FUNCTION__);
#endif
	o->listeners.insert(o->listeners.end(),this);
#ifdef USE_MMGR
	m_setOwner(__FILE__, __LINE__, __FUNCTION__);
#endif
	listening.insert(listening.end(),o);
#ifdef USE_MMGR
	m_resetGlobals();
#endif
}

void CObject::DeleteDeathDependence(CObject *o)
{
	//note that we can be listening to a single object from several different places (like curreclaim in CBuilder and lastAttacker in CUnit, grr) so we should only remove one of them
#ifdef USE_MMGR
	m_setOwner(__FILE__, __LINE__, __FUNCTION__);
#endif
	ListErase<CObject*>(listening, o);
#ifdef USE_MMGR
	m_setOwner(__FILE__, __LINE__, __FUNCTION__);
#endif
	ListErase<CObject*>(o->listeners, this);
#ifdef USE_MMGR
	m_resetGlobals();
#endif
}
